import(graphics)
import(grDevices)
import(stats)
import(utils)
import(segmented)
import(foreach)
import(doSNOW)
import(parallel)
importFrom("methods", "as", "getClass", "new")
importFrom(stats,dexp)
importFrom(stats,pexp)
importFrom(stats,qexp)
importFrom(stats,rexp)
importFrom(stats,runif)
import(survival)
import(fastmatch)

export(simdata)
export(sim_followup)
export(rpwexp)
export(rpwexp_conditional)
export(ppwexp)
export(ppwexp_conditional)
export(dpwexp)
export(qpwexp)
export(qpwexp_conditional)

export(pwexp.fit)
export(boot.pwexp.fit)
export(cv.pwexp.fit)
export(predict.pwexp.fit)
export(predict.boot.pwexp.fit)
S3method(boot.pwexp.fit, default)
S3method(boot.pwexp.fit, pwexp.fit)
S3method(predict, pwexp.fit)
S3method(predict, boot.pwexp.fit)
S3method(cv.pwexp.fit, default)
S3method(cv.pwexp.fit, pwexp.fit)

export(plot_survival)
export(plot_event)
S3method(plot, pwexp.fit)
S3method(plot, boot.pwexp.fit)
S3method(plot_survival, default)
S3method(plot_survival, pwexp.fit)
S3method(plot_survival, boot.pwexp.fit)
S3method(plot_event, default)
S3method(plot_event, predict.pwexp.fit)
S3method(plot_event, predict.boot.pwexp.fit)

export(cut_dat)
